########## HEADER ########################################################################################################  
#Our modification to LNNMV family, which allows to adapt EBarrays to process FPKM data
#Prior to running the code ensure that you have the ''EBarray'' package installed.
#In order to create the NNMV family run the whole code in this file. The family R object will be stored under name: ''nolog''
#Our modifications are listed in the comments.
#============ END of HEADER =======================================================================================================

#------------------------------------------------------------------------------------------------------------------
f0 <- function(theta, args) {
  theta[2] <- exp(theta[2])
  spnt <- args$sigmag2 + args$n * theta[2]
  negloglik <- log(cbind(rep(2 * base::pi, length(args$sigmag2)), 
                         args$sigmag2, spnt)) %*% c(args$n, args$n - 1, 1) + 
                           (args$n * theta[1] * theta[1] + args$sumsq.logdata - 
                           2 * theta[1] * args$sum.logdata)/args$sigmag2 - 
                           theta[2] * (args$sum.logdata * args$sum.logdata + 
                           args$n * args$n * theta[1] * theta[1] - 2 * args$n * 
                           theta[1] * args$sum.logdata)/(args$sigmag2 * 
                           spnt)
  -0.5 * negloglik
}
logDensity <- function() {
  cat("\n logDensity is not available for family \"NNMV\" \n")
}

#------------------------------------------------------------------------------------------------------------------
f0.arglist <- function(data, patterns, groupid) {
  #We simply got rid of logtransform of the data here, so we end up with NNMV family (Normal-Normal + Modified Variances)
  k <- nrow(data)
  n <- sum(groupid != 0)
  groups <- unique(groupid)
  groups <- groups[groups != 0]
  ngroup <- length(groups)
  vars <- 0
  for (i in 1:ngroup) {
    temp <- data[, groupid == groups[i]]
    ncoltemp <- sum(groupid == groups[i])
    if (ncoltemp == 1) {
      vars <- vars
    }
    else {
      vars <- vars + (ncoltemp - 1) * apply(temp, 1, 
                                            FUN = var)
    }
  }
  svars <- vars/(n - ngroup)
  v0 <- mean(svars)^2/((length(svars) - 1)/length(svars) * 
    var(svars)) * 2 + 4
  sigma02 <- mean(svars) * (v0 - 2)/v0
  common.args <- list(sigmag2 = (v0 * sigma02 + vars)/(n + v0 - 2))
# To get rid of inf0rmation sharing use instead: common.args <- list(sigmag2 = (vars)/(n-1))
  pattern.args <- vector("list", length(patterns))
  for (i in seq(along = patterns)) {
    pattern.args[[i]] <- vector("list", length(patterns[[i]]))
    for (j in seq(along = patterns[[i]])) {
      tmpdata <- data[, patterns[[i]][[j]], drop = FALSE]
      pattern.args[[i]][[j]] <- list(n = length(patterns[[i]][[j]]), 
                                     sum.logdata = rowSums(tmpdata), sumsq.logdata = rowSums(tmpdata * 
                                       tmpdata))
    }
  }
  list(common.args = common.args, pattern.args = pattern.args)
}
#------------------------------------------------------------------------------------------------------------------
thetaInit <- function(data) {
  data <- as.matrix(data)
  means <- rowMeans(data)
  mu.hat <- mean(means)
  tausq.hat <- var(means)
  c(mu.hat, log(tausq.hat))
}
#------------------------------------------------------------------------------------------------------------------
deflate <- function(theta, args, num.sample = 10000) {
  stop("LNNMV family does not support ordered hypotheses.")
}
#------------------------------------------------------------------------------------------------------------------
#The ne family object is stored in 'nolog'.
#usage: emfit(data, family=nolog, ... )
nolog<-new("ebarraysFamily", "LNNMV", description = "Normal-Normal with modified variances", 
       thetaInit = thetaInit, link = function(x) {
         x[2] <- log(x[2])
         x
       }, invlink = function(x) {
         
         x[2] <- exp(x[2])
         names(x) <- c("mu_0", "tao_0^2")
         x
       }, f0 = f0, deflate = deflate, f0.pp = f0, logDensity = logDensity, 
       f0.arglist = f0.arglist, lower.bound = c(-Inf, -Inf), 
       upper.bound = c(Inf, Inf))



